{************************************************************************}
PROGRAM GEODESIC;
{  The program "GEODESIC" and procedure "GEO_REDUCTION" are Copyright    }
{  1992 by Gary A. Crowell Sr.  Permission is granted for unrestricted   }
{  non-commerical use and distribution.                                  }
{       This mainline program demonstrates a very simple application of  }
{  the procedure GEO_REDUCTION, for Geodesic tracking data reduction.    }
{  It is perfectly useful as it stands, but the user may wish to add     }
{  some ruffles and lace, such as printer & file output, the ability to  }
{  tag an ID onto the data, and calculation of tracking station height   }
{  differences.  GEODESIC is written in Borland Turbo Pascal for IBM     }
{  flavor computers, but no unique extensions are used so it should be   }
{  usable with any computer/compiler combination.                        }
{************************************************************************}
VAR     BASE, EA, EE, WA, WE, LH, WH, A, C : REAL;



{************************************************************************}
PROCEDURE GEO_REDUCTION (VAR Alt, PctClose  {Results of data reduction.  }
                             : Real;
                             Baseline,      {Baseline length.            }
                             Bheight,       {Height of tracker B above A.}
                             Lheight,       {Height of launcher above A. }
                             AAzmuth, AElev,{Raw tracking data from A.   }
                             BAzmuth, BElev {Raw tracking data from B.   }
                             : Real);

{       General purpose routine for geodesic data reduction including    }
{  consideration of non-level tracking geometry.  This implementation    }
{  was developed directly from the derivation of the Geodesic equations  }
{  obtained from their author, Mike Ady, in January 1977.                }
{       The equations for the special case where the tracking system is  }
{  level are included for reference but are commented out in this        }
{  listing.  If the height differences are zero, the general equations   }
{  will reduce to the equivalent of the special level case.              }
{       These equations account for height differences in the tracking   }
{  system without requiring odd rotations of the coordinate system.  To  }
{  properly utilize these equations, the trackers >>MUST<< be            }
{  independently zeroed to the horizontal.  That is, align the azmuth on }
{  the opposite tracker, but align the elevation to the local horizontal }
{  with a bubble level.  Aligning the trackers to a non-level plane      }
{  passing through the stations and launch site is NOT appropriate.      }
{       The result is that there should be no restrictions on height     }
{  differences between trackers; one could be at the bottom of a well,   }
{  and the other on the roof of a convenient high-rise.  (You might not  }
{  get many good tracks out of the well, but when you did, the equations }
{  would work fine.                                                      }
{                                                                        }
{       Tracker A is the origin of the tracking coordinate system.       }
{  Tracker B is in the +X axis direction at an x distance equal to the   }
{  horizontal length of the baseline.  The launch site is assumed to be  }
{  displaced from the baseline in the +Y, +X quadrant.  Heights above    }
{  tracker A are in the +Z direction.                                    }
{       Actually this set of equations is not entirely general, since    }
{  they assume that tracker A is at the origin, and that tracker B is    }
{  located in the direction of the +X axis.  A truly general form would  }
{  allow both trackers to be located anywhere in the coordinate system.  }
{  A fully general form might be useful in putting together DR for a     }
{  three or four tracker system.  Any comments or suggestions regarding  }
{  this material would be appreciated.                                   }
{                                                                        }
{                                            Gary Crowell                }
{                                            71461.1525@compuserve.com   }
{                                            garyc@data-cache.com        } 
{                                                                        }
{************************************************************************}

CONST   DegRad = 57.29587;  {Constant for converting degrees to radians. }

VAR     LOSA, LOSB,         {  Line-Of-Sight lengths from trackers to    }
                            {    intersection with geodesic line.        }
        Sx, Sy, Sz,         {  Geodesic vector that connects endpoints   }
                            {    of LOS vectors A and B.                 }
        Mx, My, Mz,         {  Vector from system origin to weighted     }
                            {    midpoint of geodesic line.              }
        Ax, Ay, Az,         {  Unit vector in direction of LOSA.         }
        Bx, By, Bz,         {  Unit vector in direction of LOSB.         }
        F, InvF             {  Intermediate terms used in calculations.  }
        : REAL;

Begin

   {Conversion of raw tracking data from degrees to radians for Pascal   }
   {trig functions.                                                      }
   AAzmuth  := AAzmuth / DegRad;
   AElev    := AElev   / DegRad;
   BAzmuth  := BAzmuth / DegRad;
   BElev    := BElev   / DegRad;

   {Calculate unit vector of line-of-sight from tracker A; zero azmuth   }
   {is aligned on tracker B with positive angles swinging towards the    }
   {launch site.                                                         }
   Ax :=  COS(AElev) * COS(AAzmuth);
   Ay :=  COS(AElev) * SIN(AAzmuth);
   Az :=  SIN(AElev);

   {Calculate unit vector of line-of-sight from tracker B; zero azmuth   }
   {is aligned on tracker A with positive angles swinging towards the    }
   {launch site.                                                         }
   Bx := -COS(BElev) * COS(BAzmuth);
   By :=  COS(BElev) * SIN(BAzmuth);
   Bz :=  SIN(BElev);

   {Calculation of intermediate terms.  The following equation for F     }
   {is a simplification of the corresponding equation typically          }
   {published for this factor.                                           }
   F    :=  (Az * Bz) - COS(AElev) * COS(BElev) * COS(AAzmuth + BAzmuth);
   InvF :=  1.0 / (1.0 - SQR(F));

   {The following block of equations are the Geodesic method as          }
   {presented in the September 83 Model Rocketeer (and corrected in the  }
   {11/83 issue), with some minor simplification.  They do not account   }
   {for height differences between the tracking stations or launch site. }
   {If this block is uncommented, and the other lines below are          }
   {commented out, this block will give the same results as the general  }
   {equations for a level-case tracking system.                          }
   {  LOSB := (F * Ax - Bx) * InvF;                                      }
   {  LOSA := (Ax - F * Bx) * InvF;                                      }
   {  Alt  := LOSB * LOSA * (Az + Bz) / (LOSB + LOSA);                   }
   {  PctClose := 100.0 * SQRT(InvF) * ABS(COS(BElev) * Az *             }
   {              SIN(BAzmuth) - COS(AElev) * Bz * SIN(AAzmuth)) / Alt;  }
   {  Alt  := ROUND(Alt * Baseline);                                     }


   LOSB := (Baseline * (F * Ax - Bx) + Bheight * (F * Az - Bz)) * InvF;
   LOSA := (Baseline * (Ax - F * Bx) + Bheight * (Az - F * Bz)) * InvF;

   {Note that Mx and My are not required to simply find the altitude.    }
   {These terms are useful if you desire the ground position of the      }
   {track.                                                               }
   Mx   := (LOSB * LOSA * (Ax + Bx) + LOSA * Baseline) / (LOSA + LOSB);
   My   := (LOSB * LOSA * (Ay + By)                  ) / (LOSA + LOSB);
   Mz   := (LOSB * LOSA * (Az + Bz) + LOSA * Bheight ) / (LOSA + LOSB);

   Sx   := (LOSB * Bx + Baseline) - LOSA * Ax;
   Sy   := (LOSB * By           ) - LOSA * Ay;
   Sz   := (LOSB * Bz + Bheight ) - LOSA * Az;

   Alt  := Mz - Lheight;


   {A minor point of contention might be the appropriate altitude to use }
   {as the normalizing factor (the denominator) in the closure equation. }
   {Altitude above the launcher (used here)?  Altitude above the origin? }
   {Something else?                                                      }
   {Note also that this equation and the use of the 'S' vector do not    }
   {appear similar to the 'standard' equations.  They are identical, and }
   {this form makes it easier to see that we are talking about the       }
   {length of the geodesic line.                                         } 
   PctClose := ABS(100.0 * SQRT(SQR(Sx) + SQR(Sy) + SQR(Sz)) / Alt);


end;  {PROCEDURE GEO_REDUCTION}



{************************************************************************}
Begin {GEODESIC MAINLINE}
{************************************************************************}

   WriteLn('Program "GEODESIC" and procedure "GEO_REDUCTION" are');
   WriteLn('Copyright 1992 by Gary A. Crowell, Sr., [71461,1525].');
   WriteLn('This program is not approved for use by the NAR.');
   WriteLn;WriteLn;

   WriteLn('PRESS CTL-C TO EXIT'); WriteLn;
   WriteLn('ENTER BASELINE LENGTH : ');
   Write('    (horizontal distance between trackers) ');
   ReadLn(BASE); WriteLn;
   
   WriteLn('ENTER HEIGHT DIFFERENCE BETWEEN');
   WriteLn('   TRACKER EAST AND LAUNCH SITE : ');
   Write('    (+ if launcher is heigher than tracker) ');
   ReadLn(LH); WriteLn;
   
   WriteLn('ENTER HEIGHT DIFFERENCE BETWEEN');
   WriteLn('   TRACKER EAST AND TRACKER WEST : ');
   Write('    (+ if West is heigher than East) ');
   ReadLn(WH); WriteLn;

   Repeat {until CTL-C is pressed to exit}

     WriteLn;
     WriteLn('PRESS CTL-C TO EXIT');
     Write('ENTER EAST AZMUTH:    '); ReadLn(EA);
     Write('ENTER EAST ELEVATION: '); ReadLn(EE);
     Write('ENTER WEST AZMUTH:    '); ReadLn(WA);
     Write('ENTER WEST ELEVATION: '); ReadLn(WE);

     GEO_REDUCTION(A, C, BASE, WH, LH, EA, EE, WA, WE);

     WriteLn;
     {The final altitude should be rounded off to whole meters here,     }
     {but I haven't so that the output can be compared to the test data. }
     WriteLn('Altitude = ',A:6:1,'  Closure = ',C:6:1);

     If C > 10 Then WriteLn('>>> TRACK DID NOT CLOSE <<<');
     WriteLn; WriteLn;

   Until false;  {forever}

end.  {GEODESIC}
{************************************************************************}


{SAMPLE TEST DATA:                                                       }

{BASELINE = 300;   LAUNCHER HEIGHT = 0;   TRACKER WEST HEIGHT = 0        }

{  EAST AZ     EAST EL     WEST AZ     WEST EL      ALT     CLOSE        }
{  -------     -------     -------     -------    ------    -----        }
{     90          45          50          40       380.0     6.3         }
{     30          45          60          45       203.3    31.2         }
{    120          75          25          55       596.3     6.2         }
{     30          80          40          85      1337.8     3.2         }


{BASELINE = 300;   LAUNCHER HEIGHT = 10;  TRACKER WEST HEIGHT = 0        }

{  EAST AZ     EAST EL     WEST AZ     WEST EL      ALT     CLOSE        }
{  -------     -------     -------     -------    ------    -----        }
{     90          45          50          40       370.0     6.5         }
{     30          45          60          45       193.3    32.8         }
{    120          75          25          55       586.3     6.3         }
{     30          80          40          85      1327.8     3.2         }


{BASELINE = 300;   LAUNCHER HEIGHT = 0;   TRACKER WEST HEIGHT = 10       }

{  EAST AZ     EAST EL     WEST AZ     WEST EL      ALT     CLOSE        }
{  -------     -------     -------     -------    ------    -----        }
{     90          45          50          40       386.7     8.0         }
{     30          45          60          45       208.1    27.7         }
{    120          75          25          55       609.1     5.8         }
{     30          80          40          85      1340.4     3.3         }


{BASELINE = 300;   LAUNCHER HEIGHT = 10;  TRACKER WEST HEIGHT = 10       }

{  EAST AZ     EAST EL     WEST AZ     WEST EL      ALT     CLOSE        }
{  -------     -------     -------     -------    ------    -----        }
{     90          45          50          40       376.7     8.3         }
{     30          45          60          45       198.1    29.1         }
{    120          75          25          55       599.1     5.8         }
{     30          80          40          85      1330.4     3.3         }
